#ifndef CBRIDGE_H
#define CBRIDGE_H

#include <QObject>
#include <QQuickImageProvider>
#include "../common/sources/system/cmaths.h"

// pont entre les parties QML et C++

enum    ETabIndex
{
    eTabMap = 0,
    eTabTiles,
    eTabWalls,
    eTabObjects
};

enum    EQMLTools
{
    eQMLToolDraw = 0,
    eQMLToolSelect,
    eQMLToolPaste
};

//---------------------------------------------------------------------------------------------
class CBridge : public QObject
{
    Q_OBJECT

public:
    explicit CBridge(QObject *parent = 0);

    void updateQMLImage() {emit newFrameAvailable();}

    Q_INVOKABLE void    setMapWidth(qint32 valeur);
    Q_INVOKABLE void    setMapHeight(qint32 valeur);
    Q_INVOKABLE void    resizeMap();
    Q_INVOKABLE void    setMouse(qint32 x, qint32 y);
    Q_INVOKABLE void    setMousePress(qint32 button);
    Q_INVOKABLE void    setMouseRelease(qint32 button);
    Q_INVOKABLE void    setTabIndex(qint32 index);
    Q_INVOKABLE void    save(QString fileName);
    Q_INVOKABLE void    load(QString fileName);
    Q_INVOKABLE void    undo();
    Q_INVOKABLE void    redo();
    Q_INVOKABLE void    copy();
    Q_INVOKABLE void    cut();
    Q_INVOKABLE void    setTool(qint32 tool);

    CVec2       mMapSize;
    CVec2       mMousePos;
    ETabIndex   mTabIndex;
    EQMLTools   mTool;
    bool        leftPressing;
    bool        leftPressed;
    bool        rightPressed;
signals:
    Q_SIGNAL void newFrameAvailable();
};

//---------------------------------------------------------------------------------------------
class MyImageProvider : public QQuickImageProvider
{
public:
    MyImageProvider();
    QImage requestImage(const QString &id, QSize *size, const QSize &requestedSize);
};

extern CBridge     bridge;

#endif // CBRIDGE_H
