#ifndef CEDITOR_H
#define CEDITOR_H

#include "../common/sources/Map.h"
#include "bridge.h"
#include "clipboard.h"
#include <QUndoStack>

#define CURSOR_COLOR    QColor(0, 255, 0)
#define SELECTION_COLOR QColor(0, 255, 255)

class CEditor
{
public:
    CEditor();

    QImage      displayMap(CMap& map, CVec2 mousePos, ETabIndex tabIndex);
    EWallSide   getWallSide(CVec2 localPos);
    EWallSide   getWallSideAbs(CVec2 pos);
    void        useTool();
    void        toolCopy();
    void        toolCut();
    void        unselect();

    QUndoStack* mUndoStack;
    CClipboard  mClipboard;

private:
    void        toolDraw();
    void        toolSelect();
    void        toolPaste();
    void        drawCursor(QImage* image, CVec2 mousePos, ETabIndex tabIndex);
    void        drawSelection(QImage* image, ETabIndex tabIndex);
    void        drawTileRect(QImage* image, CVec2 mousePos, QColor color);
    void        drawWallRect(QImage* image, CVec2 mousePos, QColor color);
    void        drawObjectRect(QImage* image, CVec2 mousePos, QColor color);
    void        drawTileSelection(QImage* image);
    void        drawWallSelection(QImage* image);
    void        drawObectSelection(QImage* image);
    void        drawRectangleSelection(QImage* image, CRect rect);
    void        drawPasteSelection(QImage* image);
    CRect       getSelectionRect();
    void        copyData(CRect rect);

    CVec2       mSelectStart;
    CVec2       mSelectEnd;
};

extern CEditor  editor;

#endif // CEDITOR_H
