#include "tools.h"
#include "../common/sources/Map.h"
#include "bridge.h"
#include <QDebug>

//-------------------------------------------------------------------
drawTileTool::drawTileTool(QUndoCommand *parent)
    : QUndoCommand(parent)
{
}

bool drawTileTool::init(CVec2 pos, int type)
{
    mPos = pos / TILE_SIZE;
    CTile*   tile = map.getTile(mPos);

    if (tile == NULL)
        return false;

    if (tile->mType == type)
        return false;

    mOldType = tile->mType;
    mType = type;
    return true;
}

void drawTileTool::undo()
{
    map.setTile(mPos, mOldType);
    bridge.updateQMLImage();
}

void drawTileTool::redo()
{
    map.setTile(mPos, mType);
    bridge.updateQMLImage();
}

//-------------------------------------------------------------------
drawWallTool::drawWallTool(QUndoCommand *parent)
    : QUndoCommand(parent)
{
}

bool drawWallTool::init(CVec2 pos, int type)
{
    mPos = pos / TILE_SIZE;
    mSide = editor.getWallSideAbs(pos);

    CTile*   tile = map.getTile(mPos);

    if (tile == NULL)
        return false;

    if (tile->mWalls[mSide].mType == type)
        return false;

    mOldType = tile->mWalls[mSide].mType;
    mType = type;
    return true;
}

void drawWallTool::undo()
{
    map.setWall(mPos, mSide, mOldType);
    bridge.updateQMLImage();
}

void drawWallTool::redo()
{
    map.setWall(mPos, mSide, mType);
    bridge.updateQMLImage();
}

//-------------------------------------------------------------------
cutTool::cutTool(QUndoCommand *parent)
    : QUndoCommand(parent)
{
}

bool cutTool::init(CVec2 pos, ETabIndex tabIndex)
{
    if (editor.mClipboard.isEmpty() == true)
        return false;

    mPos = pos;
    mTabIndex = tabIndex;

    mData.copyFromClip(&editor.mClipboard);

    if (mData.isEmpty() == true)
        return false;

    return true;
}

void cutTool::undo()
{
    mData.copyToMap(mPos, mTabIndex);
    bridge.updateQMLImage();
}

void cutTool::redo()
{
    for (int y = 0; y < mData.mSize.y; ++y)
        for (int x = 0; x < mData.mSize.x; ++x)
        {
            CTile*  tile = map.getTile(CVec2(mPos.x + x, mPos.y + y));

            if (tile != NULL)
            {
                if (mTabIndex == eTabTiles)
                    tile->mType = 0;
                else if (mTabIndex == eTabWalls)
                    for (int i = 0; i < eWallSideMax; ++i)
                        tile->mWalls[i].mType = 0;
            }
        }
    bridge.updateQMLImage();
}

//-------------------------------------------------------------------
pasteTool::pasteTool(QUndoCommand *parent)
    : QUndoCommand(parent)
{
}

bool pasteTool::init(CVec2 pos, ETabIndex tabIndex)
{
    if (editor.mClipboard.isEmpty() == true)
        return false;

    mClipboard.copyFromClip(&editor.mClipboard);

    if (mClipboard.isEmpty() == true)
        return false;

    mPos = pos;
    mTabIndex = tabIndex;
    mData.copyFromMap(mPos, mClipboard.mSize);

    if (mData.isEmpty() == true)
        return false;

    return true;
}

void pasteTool::undo()
{
    mData.copyToMap(mPos, mTabIndex);
    bridge.updateQMLImage();
}

void pasteTool::redo()
{
    mClipboard.copyToMap(mPos, mTabIndex);
    bridge.updateQMLImage();
}
