﻿#include "walls.h"

CWalls   walls;

CWalls::CWalls()
{

}

// Pour chaque case, le nom de fichier et les coordonnées des graphismes pour les murs "arrière", "gauche" et "droite".
struct CDispTile
{
    char        file[16];
    uint16_t    x, y;
};

// Les cases sont triées de gauche à droite horizontallement et les "rangs" de cases sont triés du plus loin au plus proche

static const CDispTile   gDispTiles[WALL_TABLE_HEIGHT][WALL_TABLE_WIDTH][3] =
{
    // Rang 3 (le plus loin)
    {
        {{"", 0, 0}, {"", 0, 0},               {"", 0, 0}               }, // 03
        {{"", 0, 0}, {"Wall13_L.png", 8, 58},  {"", 0, 0}               }, // 13
        {{"", 0, 0}, {"Wall23_L.png", 78, 59}, {"Wall23_R.png", 134, 59}}, // 23
        {{"", 0, 0}, {"", 0, 0},               {"Wall33_R.png", 180, 58}}, // 33
        {{"", 0, 0}, {"", 0, 0},               {"", 0, 0}               }  // 43
    },

    // Rang 2
    {
        {{"Wall02_F.png",   0, 58}, {"", 0, 0},               {"", 0, 0}               }, // 02
        {{"Wall12_F.png",   7, 58}, {"Wall12_L.png",  0, 57}, {"", 0, 0}               }, // 12
        {{"Wall22_F.png",  77, 58}, {"Wall22_L.png", 60, 52}, {"Wall22_R.png", 146, 52}}, // 22
        {{"Wall32_F.png", 146, 58}, {"", 0, 0},               {"Wall32_R.png", 216, 57}}, // 32
        {{"Wall42_F.png", 216, 58}, {"", 0, 0},               {"", 0, 0}               }  // 42
    },

    // Rang 1
    {
        {{"", 0, 0},                {"", 0, 0},               {"", 0, 0}               }, // 01
        {{"Wall11_F.png",   0, 52}, {"", 0, 0},               {"", 0, 0}               }, // 11
        {{"Wall21_F.png",  59, 52}, {"Wall21_L.png", 33, 42}, {"Wall21_R.png", 164, 42}}, // 21
        {{"Wall31_F.png", 164, 52}, {"", 0, 0},               {"", 0, 0}               }, // 31
        {{"", 0, 0},                {"", 0, 0},               {"", 0, 0}               }  // 41
    },

    // Rang 0 (le plus proche)
    {
        {{"", 0, 0},                {"", 0, 0},              {"", 0, 0}               }, // 00
        {{"Wall10_F.png",   0, 42}, {"", 0, 0},              {"", 0, 0}               }, // 10
        {{"Wall20_F.png",  32, 42}, {"Wall20_L.png", 0, 33}, {"Wall20_R.png", 191, 33}}, // 20
        {{"Wall30_F.png", 191, 42}, {"", 0, 0},              {"", 0, 0}               }, // 30
        {{"", 0, 0},                {"", 0, 0},              {"", 0, 0}               }  // 40
    }
};

//--------------------------------------------------------------------------------------------------------
// renvoie l'ordre dans lequel on va parcourir la table
CVec2       CWalls::getDrawSequence(CVec2 pos)
{
    CVec2   tablePos;

    // en x: à cause de la perspective et de certains murs qui se superposent, on doit dessiner
    // d'abord les plus à gauche/droite, puis ceux du centre.
    static const int xSequence[] = {0, 4, 1, 3, 2};
    tablePos.x = xSequence[pos.x];

    // en "profondeur": on dessine d'abord les cases les plus loin, puis les plus proche.
    // C'est déjà l'ordre de la table.
    tablePos.y = pos.y;

    return tablePos;
}

//--------------------------------------------------------------------------------------------------------
// renvoie la position dans la map à partir de la position dans la table
// cette position est relative au joueur et on considère qu'on regarde vers le nord (le haut de la map)

CVec2       CWalls::getMapPos(CVec2 tablePos)
{
    return CVec2(tablePos.x - 2,
                 tablePos.y - WALL_TABLE_HEIGHT + 1);
}

//--------------------------------------------------------------------------------------------------------
// renvoie l'index dans la table pour la direction du mur donnée
int       CWalls::getTableSide(EWallSide side)
{
    if (side == eWallSideUp)
        return 0;

    if (side == eWallSideLeft)
        return 1;

    if (side == eWallSideRight)
        return 2;

    return 0;
}

//--------------------------------------------------------------------------------------------------------
const char* CWalls::getFilename(CVec2 tablePos, EWallSide side)
{
    int sideIndex = getTableSide(side);
    const char* baseName = gDispTiles[tablePos.y][tablePos.x][sideIndex].file;

    if (baseName[0] != 0)
    {
        static char fileName[256];
        sprintf(fileName, "gfx/3DView/%s", baseName);
        return fileName;
    }
    return "";
}

//--------------------------------------------------------------------------------------------------------
CVec2       CWalls::getGfxPos(CVec2 tablePos, EWallSide side)
{
    int sideIndex = getTableSide(side);
    const CDispTile*  dispTile = &gDispTiles[tablePos.y][tablePos.x][sideIndex];
    return CVec2(dispTile->x, dispTile->y);
}
